@@ -1,31 +1,34 @@
 ################################################################################
-# Copyright (c) 2006-2017 Franz Inc.  
+# Copyright (c) 2006-2017 Franz Inc.
 # All rights reserved. This program and the accompanying materials are
 # made available under the terms of the MIT License which accompanies
 # this distribution, and is available at http://opensource.org/licenses/MIT
 ################################################################################
 from __future__ import absolute_import, division, with_statement
 
+import copy
+import inspect
+import math
+import re
+import sys
+import threading
+from contextlib import contextmanager
 from datetime import timedelta
 
-from past.utils import old_div
-from past.builtins import basestring
-
+import six
 from franz.miniclient.agjson import encode_json
+from franz.openrdf.model.value import URI
 from franz.openrdf.repository.attributes import AttributeFilter
 from franz.openrdf.rio.formats import Format
-
+from past.builtins import basestring
+from past.utils import old_div
+from six import python_2_unicode_compatible
 from six.moves.urllib.parse import quote, urlparse
 
-import copy, inspect, math, re, threading
-
-from contextlib import contextmanager
-from .request import encode, serialize, decode, deserialize, \
-    jsonRequest, nullRequest, urlenc, RequestError
 from ..openrdf.util.contexts import wrap_context
 from ..openrdf.util.strings import to_native_string
-
-from six import python_2_unicode_compatible
+from .request import (RequestError, decode, deserialize, encode, jsonRequest,
+                      nullRequest, serialize, urlenc)
 
 
 def _split_proxy(proxy):
@@ -348,7 +351,8 @@ class Repository(Service):
                    urlenc(query=query, infer=infer, context=context, namedContext=namedContext,
                        planner=planner, checkVariables=checkVariables,
                        analyzeIndicesUsed=analyze, queryAnalysisTechnique=analysisTechnique,
-                       queryAnalysisTimeout=analysisTimeout) + (bindings or ""),
+                       queryAnalysisTimeout=analysisTimeout,
+                       returnQueryMetadata=True) + (bindings or ""),
                        callback=callback,
                        accept=accept)
 
@@ -361,6 +365,20 @@ class Repository(Service):
                            callback=callback,
                            accept=accept)
 
+    def evalGraphqlQuery(self, query, default_prefix, infer, namespaces, variables, aliases):
+        """Execute a GraphQL query. Returns a JSON object."""
+        api = "/graphql"
+        api_args = "&".join([ name + "=" + arg for name, arg in [
+            ("default-prefix", default_prefix),
+            ("infer", "false" if infer == False else infer),
+            ("namespaces", namespaces), ("variables", variables),
+            ("aliases", aliases)]
+            if arg
+        ])
+        if api_args:
+            api += "?" + api_args
+        return jsonRequest(self, method="POST", url=self.url+api, body=query)
+
     def definePrologFunctors(self, definitions):
         """Add Prolog functors to the environment. Takes a string
         containing Lisp-syntax functor definitions (using the <-- and
@@ -527,7 +545,7 @@ class Repository(Service):
                             innerChars=None, borderChars=None, tokenizer=None):
         """Create a free-text index. predicates, if given, should be a
         list of resources. indexLiterals can be True, False, or a list
-        of resources, indicating the literal types to index.
+        of strings, indicating the literal types or langs to index.
         indexResources can be True, False, or \"short\". indexFields
         can be a list containing any combination of the elements
         \"subject\", \"predicate\", \"object\", and \"graph\".
@@ -604,6 +622,25 @@ class Repository(Service):
     def getNamespace(self, prefix):
         return jsonRequest(self, "GET", "/namespaces/" + quote(prefix))
 
+    def clearQueryOptions(self):
+        """
+        Deletes all query options in this repository for the current user.
+        """
+        nullRequest(self, "DELETE", "/query-options")
+
+    def setQueryOption(self, name, value):
+        nullRequest(self, "PUT", "/query-options/" + quote(name),
+                    value, content_type="text/plain")
+
+    def deleteQueryOption(self, name):
+        nullRequest(self, "DELETE", "/query-options/" + quote(name))
+
+    def listQueryOptions(self):
+        return jsonRequest(self, "GET", "/query-options")
+
+    def getQueryOption(self, name):
+        return jsonRequest(self, "GET", "/query-options/" + quote(name))
+
     def listMappedTypes(self):
         return jsonRequest(self, "GET", "/mapping/type")
 
@@ -750,7 +787,7 @@ class Repository(Service):
         nullRequest(self, "PUT", "/warmup?" + urlenc(includeStrings=includeStrings,
                                                      includeTriples=includeTriples,
                                                      index=indices))
-           
+
     def openSession(self, autocommit=False, lifetime=None, loadinitfile=False):
         if self.sessionAlive:
             # A session is already active.  Do nothing.
@@ -990,6 +1027,101 @@ class Repository(Service):
         return jsonRequest(
             self, 'DELETE', '/attributes/definitions?' + urlenc(name=name))
 
+    def loadDocument(self, doc, doc_format, base=None,
+                     rules=None, subject=None, prefix=None,
+                     rename=None, rdf_type=None, lang=None, skip=None,
+                     transform=None, graph=None, json_store_source=None,
+                     csv_columns=None, csv_separator=None, csv_quote=None,
+                     csv_whitespace=None, csv_double_quote=None, csv_escape=None,
+                     attributes=None, commit=None, context=None,
+                     encoding=None, content_encoding=None):
+        mime = Format.mime_type_for_format(doc_format)
+        if encoding:
+            mime += ';charset=' + encoding
+
+        if prefix is None:
+            prefix = {}
+        if rename is not None:
+            # Copy the keys, since we will be changing some renames
+            for k, v in list(six.iteritems(rename)):
+                if isinstance(v, URI):
+                    # To provide a full URI for a key we need to use both 'prefix'
+                    # and 'rename' options
+                    namespace, local_name = v.split()
+                    if local_name != k:
+                        rename[k] = local_name
+                    prefix[k] = namespace
+
+        prefix_list = uri_dict_to_string_list(prefix) or []
+        if base is not None:
+            prefix_list.append(uri_to_string(base))
+
+        args = {
+            'transform-rules': rules,
+            'tr-id': uri_to_string(subject),
+            'tr-prefix': prefix_list,
+            'tr-rename': dict_to_string_list(rename),
+            'tr-type': uri_dict_to_string_list(rdf_type),
+            'tr-lang':  dict_to_string_list(lang),
+            'tr-skip': skip,
+            'tr-transform':  dict_to_string_list(transform),
+            'tr-graph': ['%s=%s' % (k, URI(str(v)))
+                         for k, v in six.iteritems(graph or {})],
+            'json-store-source': bool(json_store_source),
+            'csv-columns': ','.join(csv_columns) if csv_columns else None,
+            'csv-separator': csv_separator,
+            'csv-quote': csv_quote,
+            'csv-whitespace': ','.join(csv_whitespace) if csv_whitespace is not None else None,
+            'csv-double-quote-p': bool(csv_double_quote) if csv_double_quote is not None else None,
+            'csv-escape': csv_escape,
+            'context': str(URI(str(context))) if context else None,
+            'commit': commit,
+            'attributes': attributes and encode_json(attributes)
+        }
+        return nullRequest(self, 'POST', '/statements/transform?' + urlenc(**args),
+                           doc, content_type=mime, content_encoding=content_encoding)
+
+    def getGeneration(self):
+        return jsonRequest(self, 'GET', '/generation')
+
+
+def uri_to_string(uri):
+    """
+    Converts a URI to a string. If the input is None then None is returned.
+    If the input is a string it is returned after stripping angle brackets
+    (if present).
+    :param uri: Either a URI object or a string.
+    :type uri: str|URI|None
+    :return: str|None
+    """
+    if uri is None:
+        return None
+    elif isinstance(uri, URI):
+        return uri.uri
+    else:
+        return URI(uri).uri
+
+
+def uri_dict_to_string_list(d):
+    """
+    Convert a dict to a list of strings of the form key=value,
+    but make sure that URI objects are serialized without angle brackets.
+    """
+    if d is None:
+        return None
+    result = []
+    for k, v in six.iteritems(d):
+        result.append('%s=%s' % (uri_to_string(k), uri_to_string(v)))
+    return result
+
+
+def dict_to_string_list(d):
+    """
+    Converts a dict to a list of 'key=value' strings. If the input is None
+    then None will be returned.
+    """
+    return ['%s=%s' % kv for kv in six.iteritems(d)] if d else None
+
 
 def time_in_seconds(t):
     """
